
/****** bisection.c *****************************************************/
/*									*/
/*   Included in this module are recursive coordinate bisection 	*/
/*   programs and recursive inertial bisection programs.		*/
/*									*/
/*   Recursive Coordinate Bisection:					*/
/*	CoorBisecMap:	   returns mapping array and does not move data */
/*	CoorWeighBisecMap: weighted version of CoorBisecMap		*/
/*	CoorBisec:	   moves data and updates indexes of local data	*/
/*	CoorWeighBisec:	   weighted version of CoorBisec		*/
/*									*/
/*   Recursive Inertial Bisection:					*/
/*	InerBisecMap:	   returns mapping array and does not move data */
/*	InerWeighBisecMap: weighted version of InerBisecMap		*/
/*	InerBisec:	   moves data and updates indexes of local data */
/*	InerWeighBisec:	   weighted version of InerBisec		*/
/*   									*/
/*************************************************** Shin 11/23/93 ******/

#include <stdio.h>
#include "parti.h"
#include "bisec.h"
#include "vararg.h"

#define	 MAPLEVEL	5	/* when level >= MAPLEVEL, the calls to
				   CoorBisecMap() will be changed to
				   CoorBisec() to reduce execution time */

/* 
 * Function:
 * 	Unweighted Recursive Coordinate Bisection
 * 
 * Synopsis:
 *	CoorBisecMap (maparray, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
CoorBisecMap VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    ndata    = (int) va_arg (ap, int);
    ndim     = (int) va_arg (ap, int);
    data     = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	RecCoorBisecMap (level, maparray, ndata, ndim, data);
    else
	RecCoorBisecRemap (level, maparray, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Coordinate Bisection (called from Fortran)
 * 
 * Synopsis:
 *	CoorBisecMap (maparray, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
coorbisecmap_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    ndata    = (int *) va_arg (ap, int *);
    ndim     = (int *) va_arg (ap, int *);
    data     = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	RecCoorBisecMap (level, maparray, *ndata, *ndim, data);
    else
	RecCoorBisecRemap (level, maparray, *ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Coordinate Bisection
 * 
 * Synopsis:
 *	CoorWeighBisecMap (maparray, load, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*load;			loads of elements
 *	int	ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
CoorWeighBisecMap VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *load;		/* loads of elements */
    int	   ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    load     = (int *) va_arg (ap, int *);
    ndata    = (int) va_arg (ap, int);
    ndim     = (int) va_arg (ap, int);
    data     = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
	WeighRecCoorBisecMap (level, maparray, load, ndata, ndim, data);
    else
	WeighRecCoorBisecRemap (level, maparray, load, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Coordinate Bisection (called from Fortran)
 * 
 * Synopsis:
 *	CoorWeighBisecMap (maparray, load, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
coorweighbisecmap_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    load     = (int *) va_arg (ap, int *);
    ndata    = (int *) va_arg (ap, int *);
    ndim     = (int *) va_arg (ap, int *);
    data     = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
	WeighRecCoorBisecMap (level, maparray, load, *ndata, *ndim, data);
    else
	WeighRecCoorBisecRemap (level, maparray, load, *ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Coordinate Bisection
 * 
 * Synopsis:
 *	CoorBisec (remaplevel, myindex, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 *
 * Note:
 *	The only differece between CoorBisecMap() and CoorBisec() is that
 *	CoorBisecMap() returns mapping array of elements and does not move 
 *	the coordinate arrays, whereas CoorBisec() moves coordinate arrays
 *	and updates the indexes of local elements.
 */
void
CoorBisec VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int) VA_START (ap, args, int);
    myindex    = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int) va_arg (ap, int);
    data       = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    RecCoorBisec (level, remaplevel, myindex, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Coordinate Bisection (called from Fortran)
 * 
 * Synopsis:
 *	CoorBisec (remaplevel, myindex, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
coorbisec_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int *) VA_START (ap, args, int *);
    myindex    = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int *) va_arg (ap, int *);
    data       = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    RecCoorBisec (level, *remaplevel, myindex, ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Coordinate Bisection
 * 
 * Synopsis:
 *	CoorWeighBisec (remaplevel, myindex, load, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
CoorWeighBisec VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int) VA_START (ap, args, int);
    myindex    = (int *) va_arg (ap, int *);
    load       = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int) va_arg (ap, int);
    data       = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    WeighRecCoorBisec (level, remaplevel, myindex, load, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Coordinate Bisection (called from Fortran)
 * 
 * Synopsis:
 *	CoorWeighBisec (remaplevel, myindex, load, ndata, ndim, x [, arg...])
 *	int	*remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
coorweighbisec_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int *) VA_START (ap, args, int *);
    myindex    = (int *) va_arg (ap, int *);
    load       = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int *) va_arg (ap, int *);
    data       = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    WeighRecCoorBisec (level, *remaplevel, myindex, load, ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Inertial Bisection
 * 
 * Synopsis:
 *	InerBisecMap (maparray, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
InerBisecMap VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    ndata    = (int) va_arg (ap, int);
    ndim     = (int) va_arg (ap, int);
    data     = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	RecInerBisecMap (level, maparray, ndata, ndim, data);
    else
	RecInerBisecRemap (level, maparray, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Inertial Bisection (called from Fortran)
 * 
 * Synopsis:
 *	InerBisecMap (maparray, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
inerbisecmap_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    ndata    = (int *) va_arg (ap, int *);
    ndim     = (int *) va_arg (ap, int *);
    data     = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	RecInerBisecMap (level, maparray, *ndata, *ndim, data);
    else
	RecInerBisecRemap (level, maparray, *ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Inertial Bisection
 * 
 * Synopsis:
 *	InerWeighBisecMap (maparray, load, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*load;			loads of elements
 *	int	ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
InerWeighBisecMap VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *load;		/* loads of elements */
    int	   ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    load     = (int *) va_arg (ap, int *);
    ndata    = (int) va_arg (ap, int);
    ndim     = (int) va_arg (ap, int);
    data     = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	WeighRecInerBisecMap (level, maparray, load, ndata, ndim, data);
    else
	WeighRecInerBisecRemap (level, maparray, load, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Inertial Bisection (called from Fortran)
 * 
 * Synopsis:
 *	InerWeighBisecMap (maparray, load, ndata, ndim, x [, arg...])
 *	int	*maparray;		result of partitioning
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
inerweighbisecmap_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *maparray;		/* result of partitioning */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    maparray = (int *) VA_START (ap, args, int *);
    load     = (int *) va_arg (ap, int *);
    ndata    = (int *) va_arg (ap, int *);
    ndim     = (int *) va_arg (ap, int *);
    data     = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    if (level < MAPLEVEL)
    	WeighRecInerBisecMap (level, maparray, load, *ndata, *ndim, data);
    else
	WeighRecInerBisecRemap (level, maparray, load, *ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Inertial Bisection
 * 
 * Synopsis:
 *	InerBisec (remaplevel, myindex, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 *
 * Note:
 *	The only differece between InerBisecMap() and InerBisec() is that
 *	InerBisecMap() returns mapping array of elements and does not move 
 *	the coordinate arrays, whereas InerBisec() moves coordinate arrays
 *	and updates the indexes of local elements.
 */
void
InerBisec VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int) VA_START (ap, args, int);
    myindex    = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int) va_arg (ap, int);
    data       = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    RecInerBisec (level, remaplevel, myindex, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Unweighted Recursive Inertial Bisection (called from Fortran)
 * 
 * Synopsis:
 *	InerBisec (remaplevel, myindex, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
inerbisec_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int *) VA_START (ap, args, int *);
    myindex    = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int *) va_arg (ap, int *);
    data       = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    RecInerBisec (level, *remaplevel, myindex, ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Inertial Bisection
 * 
 * Synopsis:
 *	InerWeighBisec (remaplevel, myindex, load, ndata, ndim, x [, arg...])
 *	int	remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
InerWeighBisec VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elements */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int) VA_START (ap, args, int);
    myindex    = (int *) va_arg (ap, int *);
    load       = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int) va_arg (ap, int);
    data       = (double **) malloc (sizeof(double *) * ndim);
    for (no = 0; no < ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    WeighRecInerBisec (level, remaplevel, myindex, load, ndata, ndim, data);

    /* free allocated memory */
    free (data);
}

/* 
 * Function:
 * 	Weighted Recursive Inertial Bisection (called from Fortran)
 * 
 * Synopsis:
 *	InerWeighBisec (remaplevel, myindex, load, ndata, ndim, x [, arg...])
 *	int	*remaplevel;		remap arrays every remaplevel levels
 *	int	*myindex;		indexes of local elements
 *	int	*load;			loads of elements
 *	int	*ndata;			number of elements
 *	int	*ndim;			number of dimensions
 *	double	*x [, arg...];		coordinate arrays
 */
void
inerweighbisec_ VA_DCL (void *, args)

{
    int	   no;
    int    level;		/* partition factor */
    int	   *remaplevel;		/* remapping level */
    int	   *myindex;		/* indexes of local elelemts */
    int	   *load;		/* loads of elements */
    int	   *ndata;		/* number of elements */
    int	   *ndim;		/* number of dimensions */
    double **data;		/* coordinate arrays */
    va_list ap;

    /* retrieve the parameters from the list */
    remaplevel = (int *) VA_START (ap, args, int *);
    myindex    = (int *) va_arg (ap, int *);
    load       = (int *) va_arg (ap, int *);
    ndata      = (int *) va_arg (ap, int *);
    ndim       = (int *) va_arg (ap, int *);
    data       = (double **) malloc (sizeof(double *) * (*ndim));
    for (no = 0; no < *ndim; no++)
	data[no]   = (double *) va_arg (ap, double *);

    /* find the level of recursions */
    for (no = PARTI_numnodes, level = -1; no; level++)
        no >>= 1;

    /* call the recursive coordinate bisection */
    WeighRecInerBisec (level, *remaplevel, myindex, load, ndata, *ndim, data);

    /* free allocated memory */
    free (data);
}
